import boto3
import botocore.client
import botocore.config
import collections
import types
from _typeshed import Incomplete
from builtins import object
from datarobot_storage.base import SeekableKeyInterface as SeekableKeyInterface, SeekableStorage as SeekableStorage, Storage as Storage, StorageGenerator as StorageGenerator
from datarobot_storage.enums import S3ServerSideEncryption as S3ServerSideEncryption
from datarobot_storage.exceptions import wrap_client_error as wrap_client_error
from datarobot_storage.put_generator import PutGenerator as PutGenerator
from datarobot_storage.utils import ClientPool as ClientPool, StorageUtilsMixin as StorageUtilsMixin, cached_property as cached_property, retry as retry
from mypy_boto3_s3.service_resource import Bucket, Object, S3ServiceResource
from typing import Any, Callable, Iterable
from typing_extensions import Self

logger: Incomplete
NA: Incomplete

class S3InternalError(Exception): ...

class S3Key(SeekableKeyInterface):
    def __init__(self, key: Object) -> None: ...
    @property
    def boto_key(self): ...
    @property
    def name(self): ...
    @property
    def size(self): ...
    @property
    def last_modified(self): ...
    def delete(self): ...
    def get_generator_factory(self, chunk_size=...) -> Callable: ...
    def get_range(self, offset: int, size: int) -> bytes: ...

class Boto3MultiPartUploader:
    parts: Incomplete
    client: Incomplete
    sha256_kwargs: Incomplete
    mp: Incomplete
    def __init__(self, bucket, s3_config, key_name: str) -> None: ...
    def complete_upload_with_retry(self): ...
    def complete_upload(self) -> None: ...
    def upload_part(self, file_object, part_number) -> None: ...
    def abort_multipart_upload(self) -> None: ...

class S3Configuration:
    aws_access_key_id: str | None
    aws_secret_access_key: str | None
    aws_session_token: str | None
    aws_profile: str | None
    aws_ca_bundle: str | None
    s3_role_arn: str | None
    s3_host: str
    s3_port: int
    s3_bucket: str
    s3_region: str
    s3_service: str
    s3_is_secure: bool
    s3_validate_certs: bool
    s3_addressing_style: str
    sse: str
    sse_kms_key_id: str
    boto_http_socket_timeout: int
    boto_num_retries: int
    boto_metadata_service_timeout: int
    boto_metadata_service_num_attempts: int
    botocore_extra_ciphers: str
    local_data_directory: str
    prefix: str
    multipart_upload_enabled: bool
    multipart_download_enabled: bool
    use_sha256: bool
    def __init__(self, aws_access_key_id: str | None = None, aws_secret_access_key: str | None = None, aws_session_token: str | None = None, aws_profile: str | None = None, aws_ca_bundle: str | None = None, s3_role_arn: str | None = None, s3_host: str = '', s3_port: int = 443, s3_bucket: str = '', s3_region: str = '', s3_service: str = '', s3_is_secure: bool = True, s3_validate_certs: bool = True, s3_addressing_style: str = 'auto', s3_server_side_encryption: str = ..., s3_server_side_encryption_key_id: str = '', boto_http_socket_timeout: int = 150, boto_num_retries: int = 7, boto_metadata_service_timeout: int = 2, boto_metadata_service_num_attempts: int = 7, botocore_extra_ciphers: str = '', local_data_directory: str = '', prefix: str = '', multipart_upload_enabled: bool = True, multipart_download_enabled: bool = True, use_sha256: bool = False) -> None: ...
    @staticmethod
    def get_int(mapping, key) -> int | object: ...
    @staticmethod
    def get_bool(mapping, key) -> bool | object: ...
    @classmethod
    def from_dict(cls, _dict: collections.abc.Mapping) -> Self: ...
    @classmethod
    def from_environ(cls) -> Self: ...
    @property
    def s3_encryption_type(self) -> str: ...
    @property
    def encrypt_key(self) -> bool: ...
    @property
    def server_side_encryption_kwargs(self) -> dict[str, str]: ...
    @property
    def sha256_kwargs(self): ...
    @property
    def endpoint_url(self) -> str: ...

class Boto3Driver:
    CHUNK_SIZE: Incomplete
    MultipartUploader = Boto3MultiPartUploader
    pagination_config: dict[str, Any]
    s3_config: Incomplete
    def __init__(self, s3_config) -> None: ...
    def patch_libraries(self) -> None: ...
    def bucket_multipart_upload(self, bucket, key_name, filename, chunk_size, callback, logging_info) -> None: ...
    def create_session(self) -> boto3.session.Session: ...
    @property
    def boto_config(self) -> botocore.config.Config: ...
    def create_connection(self) -> S3ServiceResource: ...
    @staticmethod
    def get_bucket(connection: S3ServiceResource, bucket_name: str) -> Bucket: ...
    @staticmethod
    def bucket_get_key(bucket: Bucket, key_name: str, load_key: bool = True) -> S3Key | None: ...
    def bucket_key_exists(self, bucket, key_name) -> bool: ...
    def bucket_list(self, bucket, prefix: Incomplete | None = None, delimiter: Incomplete | None = None, recursive: bool = False): ...
    def bucket_list_generator(self, bucket, prefix: Incomplete | None = None, delimiter: Incomplete | None = None) -> Iterable[S3Key]: ...
    def bucket_download(self, bucket: Bucket, key_name: str, filename: str, transfer_config: Incomplete | None = None, callback: Incomplete | None = None) -> None: ...
    def bucket_multipart_download(self, bucket: Bucket, key_name: str, filename: str, chunk_size: int, callback: Callable) -> None: ...
    def bucket_generator_download(self, key): ...
    def bucket_upload(self, bucket: Bucket, key_name: str, filename: str) -> None: ...
    def bucket_delete_key(self, bucket: Bucket, key_name: str): ...
    def bucket_delete_key_batch(self, bucket: Bucket, key_name_list: list[str]) -> None: ...
    def bucket_copy_key(self, bucket: Bucket, dest_key_name: str, source_key_name: str) -> None: ...
    def bucket_generate_presigned_url(self, bucket: Bucket, key_name: str, lifetime: int) -> str | None: ...

class bucket_from_pool:
    bucket_pool: Incomplete
    MAX_RETRY_ATTEMPTS: int
    storage: Incomplete
    bucket: Incomplete
    use_bucket_pool: Incomplete
    def __init__(self, storage) -> None: ...
    def __enter__(self) -> Bucket: ...
    def __exit__(self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: types.TracebackType) -> None: ...

class S3Storage(Storage, StorageUtilsMixin):
    MULTIPART_CHUNK_SIZE: int
    config: S3Configuration
    prefix: str
    local_data_directory: str
    driver: Boto3Driver
    get_cb: Callable
    put_cb: Callable
    use_bucket_pool: bool
    def __init__(self, bucket: str | None = None, use_bucket_pool: bool = False, get_cb: Callable | None = None, put_cb: Callable | None = None, storage_config: S3Configuration | None = None) -> None: ...
    def get_key_name(self, name) -> str: ...
    @property
    def client(self) -> botocore.client.BaseClient: ...
    @property
    def bucket_name(self) -> str: ...
    def bucket(self) -> Bucket: ...
    def get_bucket(self) -> Bucket: ...
    def download_handler(self, transmitted: int, total: int) -> None: ...
    def upload_handler(self, offset: int, nbytes: int, total: int) -> None: ...
    def exists(self, name: str) -> bool: ...
    def list(self, path: str, recursive: bool = False) -> list[str]: ...
    def get_large(self, name: str, temp_filename: str) -> bool: ...
    def get_seekable(self, name: str) -> SeekableStorage: ...
    def get(self, name: str, temp_filename: str) -> bool: ...
    def get_generator(self, name: str, **kwargs) -> StorageGenerator: ...
    def put_large(self, name: str, source_path: str) -> bool: ...
    def put(self, name: str, local_filename: str) -> bool: ...
    def put_generator(self, name: str, source: PutGenerator) -> bool: ...
    def copy(self, name: str, new_name: str) -> bool: ...
    def delete(self, name: str) -> bool: ...
    def url(self, name: str, expires_in: int | None = 600) -> str | None: ...
    def file_size(self, name: str) -> int: ...
    def delete_batch(self, name_iter: Iterable, batch_size: int | None = 1000) -> None: ...
    def delete_all(self, name: str) -> None: ...
    def last_modified_get_downloadable_package_info(self, name: str): ...
    def list_keys_by_prefix(self, prefix: str): ...
    def root_location(self) -> str: ...
